#!/usr/bin/env python

"""Measure the MTU between two IPv6 non-measurement machines."""

__author__ = 'Jeffrey Knockel'
__copyright__ = 'Copyright 2013 Jeffrey Knockel'
__license__ = 'GPL v2'

import sys

from mtuecho6 import test_mtu as test_mtu_echo
from nslookup6 import lookup_or_die
from scanerror import ScanError
from sniffing import start_sniffing

ALPHA = 0.01

def find_mtu(sniffer, addr1, addr2):
    """Find the MTU from 'addr1' to 'addr2' using binary search over tests on
    specific MTU's.
    """
    initial = lo = 1280
    hi = lo
    while True:
        sys.stderr.write('Trying %d...\n' % hi)
        ir = test_mtu_echo(sniffer, addr1, addr2, hi)
        sys.stderr.write('%s\n' % ir)
        if ALPHA <= ir.pvalues[0] <= 1.0 - ALPHA:
            raise ScanError('The \'from\' host\'s traffic too noisy or lossy--try another host?')
        if ir.pvalues[0] > 0.5:
            break
        lo = hi + 1
        if hi >= 65535:
            raise ScanError('MTU seems impossibly high--does the first host have a globally incrementing IP id and reply to ICMP echo requests?')
        hi = min(65535, (hi - 40) * 2 + 40)
    while lo < hi:
        mid = (lo + hi) // 2
        sys.stderr.write('Trying %d...\n' % mid)
        ir = test_mtu_echo(sniffer, addr1, addr2, mid)
        sys.stderr.write('%s\n' % ir)
        if ALPHA <= ir.pvalues[0] <= 1.0 - ALPHA:
            raise ScanError('The \'from\' host\'s traffic too noisy or lossy--try another host?')
        if ir.pvalues[0] > 0.5:
            hi = mid
        else:
            lo = mid + 1
    if lo <= initial:
        raise ScanError('MTU seems impossibly low')
    return lo - 1

def usage(code=1):
    sys.stderr.write('Usage: %s HOST1 HOST2\n' % sys.argv[0])
    exit(code)

def main():
    try:
        if len(sys.argv) != 3:
            usage()
        host1 = sys.argv[1]
        host2 = sys.argv[2]
        addr1 = lookup_or_die(host1)[0]
        addr2 = lookup_or_die(host2)[0]
        mtu = start_sniffing(find_mtu, True, args=(addr1, addr2))
        print mtu
    except ScanError as e:
        sys.stderr.write('Error performing scan: %s\n' % e)
    except KeyboardInterrupt:
        sys.stderr.write('Received SIGINT\n')

if __name__ == '__main__':
    main()
